@@ -283,7 +283,7 @@ gst_gl_image_sink_bin_class_init (GstGLImageSinkBinClass * klass)
       g_param_spec_enum ("output-multiview-downmix-mode",
           "Mode for mono downmixed output",
           "Output anaglyph type to generate when downmixing to mono",
-          GST_TYPE_GL_STEREO_DOWNMIX_MODE_TYPE, DEFAULT_MULTIVIEW_DOWNMIX,
+          GST_TYPE_GL_STEREO_DOWNMIX, DEFAULT_MULTIVIEW_DOWNMIX,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gst_video_overlay_install_properties (gobject_class, PROP_BIN_LAST);
@@ -694,7 +694,7 @@ gst_glimage_sink_class_init (GstGLImageSinkClass * klass)
       g_param_spec_enum ("output-multiview-downmix-mode",
           "Mode for mono downmixed output",
           "Output anaglyph type to generate when downmixing to mono",
-          GST_TYPE_GL_STEREO_DOWNMIX_MODE_TYPE, DEFAULT_MULTIVIEW_DOWNMIX,
+          GST_TYPE_GL_STEREO_DOWNMIX, DEFAULT_MULTIVIEW_DOWNMIX,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gst_video_overlay_install_properties (gobject_class, PROP_LAST);
@@ -893,8 +893,7 @@ gst_glimage_sink_get_property (GObject * object, guint prop_id,
       g_value_set_enum (value, glimage_sink->mview_downmix_mode);
       break;
     default:
-      if (!gst_video_overlay_set_property (object, PROP_LAST, prop_id, value))
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
@@ -1378,6 +1377,19 @@ configure_display_from_info (GstGLImageSink * glimage_sink,
   return TRUE;
 }
 
+static gboolean
+_mview_modes_are_equal (GstVideoMultiviewMode a, GstVideoMultiviewMode b)
+{
+  if (a == b)
+    return TRUE;
+  if (a == GST_VIDEO_MULTIVIEW_MODE_NONE && b == GST_VIDEO_MULTIVIEW_MODE_MONO)
+    return TRUE;
+  if (a == GST_VIDEO_MULTIVIEW_MODE_MONO && b == GST_VIDEO_MULTIVIEW_MODE_NONE)
+    return TRUE;
+
+  return FALSE;
+}
+
 /* Called with GST_GLIMAGE_SINK lock held, to
  * copy in_info to out_info and update out_caps */
 static gboolean
@@ -1397,7 +1409,7 @@ update_output_format (GstGLImageSink * glimage_sink)
 
   mv_mode = GST_VIDEO_INFO_MULTIVIEW_MODE (&glimage_sink->in_info);
 
-  if (glimage_sink->mview_output_mode != mv_mode) {
+  if (!_mview_modes_are_equal (glimage_sink->mview_output_mode, mv_mode)) {
     /* Input is multiview, and output wants a conversion - configure 3d converter now,
      * otherwise defer it until either the caps or the 3D output mode changes */
     gst_video_multiview_video_info_change_mode (out_info,
@@ -1418,13 +1430,37 @@ update_output_format (GstGLImageSink * glimage_sink)
   ret = configure_display_from_info (glimage_sink, out_info);
 
   if (glimage_sink->convert_views) {
+    gint new_w, new_h;
+    gint par_n, par_d;
+
     /* Match actual output window size for pixel-aligned output,
      * even though we can't necessarily match the starting left/right
-     * view parity properly */
-    glimage_sink->out_info.width = MAX (1, glimage_sink->display_rect.w);
-    glimage_sink->out_info.height = MAX (1, glimage_sink->display_rect.h);
-    GST_LOG_OBJECT (glimage_sink, "Set 3D output scale to %d,%d",
-        glimage_sink->display_rect.w, glimage_sink->display_rect.h);
+     * view parity properly for line-by-line modes, because that
+     * depends on the window being placed correctly.
+     * FIXME: Should this rescaling be configurable? */
+    if (glimage_sink->display_rect.w > 0 && glimage_sink->display_rect.h > 0) {
+      new_w = glimage_sink->display_rect.w;
+      new_h = glimage_sink->display_rect.h;
+      if (new_w != out_info->width || new_h != out_info->height) {
+        /* Recalculate PAR if rescaling */
+        gint from_dar_n, from_dar_d;
+        if (!gst_util_fraction_multiply (out_info->width, out_info->height,
+                out_info->par_n, out_info->par_d, &from_dar_n,
+                &from_dar_d) ||
+            !gst_util_fraction_multiply (from_dar_n, from_dar_d, new_h, new_w,
+                &par_n, &par_d)) {
+          par_n = glimage_sink->par_n;
+          par_d = glimage_sink->par_d;
+        }
+        out_info->par_n = par_n;
+        out_info->par_d = par_d;
+        out_info->width = new_w;
+        out_info->height = new_h;
+      }
+
+      GST_LOG_OBJECT (glimage_sink, "Set 3D output scale to %dx%d PAR %d/%d",
+          out_info->width, out_info->height, out_info->par_n, out_info->par_d);
+    }
   }
 
   s = gst_caps_get_structure (glimage_sink->in_caps, 0);
@@ -1978,10 +2014,17 @@ gst_glimage_sink_thread_init_redisplay (GstGLImageSink * gl_sink)
       GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY,
       gst_gl_shader_string_vertex_mat4_vertex_transform);
   if (gl_sink->texture_target == GST_GL_TEXTURE_TARGET_EXTERNAL_OES) {
+    gchar *frag_str;
+    frag_str =
+        gst_gl_shader_string_fragment_external_oes_get_default
+        (gl_sink->context, GST_GLSL_VERSION_NONE,
+        GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY);
+
     frag_stage = gst_glsl_stage_new_with_string (gl_sink->context,
         GL_FRAGMENT_SHADER, GST_GLSL_VERSION_NONE,
-        GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY,
-        gst_gl_shader_string_fragment_external_oes_default);
+        GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY, frag_str);
+
+    g_free (frag_str);
   } else {
     frag_stage = gst_glsl_stage_new_default_fragment (gl_sink->context);
   }
@@ -2144,6 +2187,16 @@ gst_glimage_sink_on_resize (GstGLImageSink * gl_sink, gint width, gint height)
     GST_DEBUG_OBJECT (gl_sink, "GL output area now %u,%u %ux%u",
         gl_sink->display_rect.x, gl_sink->display_rect.y,
         gl_sink->display_rect.w, gl_sink->display_rect.h);
+  } else {
+    gint viewport_dims[4];
+
+    /* save the viewport for use later */
+    gl->GetIntegerv (GL_VIEWPORT, viewport_dims);
+
+    gl_sink->display_rect.x = viewport_dims[0];
+    gl_sink->display_rect.y = viewport_dims[1];
+    gl_sink->display_rect.w = viewport_dims[2];
+    gl_sink->display_rect.h = viewport_dims[3];
   }
   GST_GLIMAGE_SINK_UNLOCK (gl_sink);
 }
@@ -2191,6 +2244,10 @@ gst_glimage_sink_on_draw (GstGLImageSink * gl_sink)
   gst_gl_context_clear_shader (gl_sink->context);
   gl->BindTexture (gl_target, 0);
 
+  if (!gst_gl_window_controls_viewport (window))
+    gl->Viewport (gl_sink->display_rect.x, gl_sink->display_rect.y,
+        gl_sink->display_rect.w, gl_sink->display_rect.h);
+
   sample = gst_sample_new (gl_sink->stored_buffer[0],
       gl_sink->out_caps, &GST_BASE_SINK (gl_sink)->segment, NULL);
   g_signal_emit (gl_sink, gst_glimage_sink_signals[CLIENT_DRAW_SIGNAL], 0,
@@ -2213,7 +2270,14 @@ gst_glimage_sink_on_draw (GstGLImageSink * gl_sink)
 
     if (gl_sink->ignore_alpha) {
       gl->BlendColor (0.0, 0.0, 0.0, alpha);
-      gl->BlendFunc (GL_SRC_ALPHA, GL_CONSTANT_COLOR);
+      if (gl->BlendFuncSeparate) {
+        gl->BlendFuncSeparate (GL_SRC_ALPHA, GL_CONSTANT_COLOR, GL_ONE,
+            GL_ONE_MINUS_SRC_ALPHA);
+      } else {
+        /* we don't have separate blend modes, perform something close to
+         * correct instead */
+        gl->BlendFunc (GL_SRC_ALPHA, GL_CONSTANT_COLOR);
+      }
       gl->BlendEquation (GL_FUNC_ADD);
       gl->Enable (GL_BLEND);
     }
@@ -2255,7 +2319,8 @@ gst_glimage_sink_on_draw (GstGLImageSink * gl_sink)
 
     if (gl->GenVertexArrays)
       gl->BindVertexArray (0);
-    _unbind_buffer (gl_sink);
+    else
+      _unbind_buffer (gl_sink);
 
     if (gl_sink->ignore_alpha)
       gl->Disable (GL_BLEND);
