@@ -19,23 +19,23 @@
 
 /**
  * SECTION:gstaudioringbuffer
+ * @title: GstAudioRingBuffer
  * @short_description: Base class for audio ringbuffer implementations
  * @see_also: #GstAudioBaseSink, #GstAudioSink
  *
- * <refsect2>
- * <para>
  * This object is the base class for audio ringbuffers used by the base
  * audio source and sink classes.
- * </para>
- * <para>
+ *
  * The ringbuffer abstracts a circular buffer of data. One reader and
  * one writer can operate on the data from different threads in a lockfree
  * manner. The base class is sufficiently flexible to be used as an
  * abstraction for DMA based ringbuffers as well as a pure software
  * implementations.
- * </para>
- * </refsect2>
+ *
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <string.h>
 
@@ -275,8 +275,8 @@ gst_audio_ring_buffer_parse_caps (GstAudioRingBufferSpec * spec, GstCaps * caps)
     info.bpf = 4;
   } else if (g_str_equal (mimetype, "audio/mpeg") &&
       gst_structure_get_int (structure, "mpegaudioversion", &i) &&
-      (i == 1 || i == 2)) {
-    /* Now we know this is MPEG-1 or MPEG-2 (non AAC) */
+      (i == 1 || i == 2 || i == 3)) {
+    /* Now we know this is MPEG-1, MPEG-2 or MPEG-2.5 (non AAC) */
     /* extract the needed information from the cap */
     if (!(gst_structure_get_int (structure, "rate", &info.rate)))
       goto parse_error;
@@ -355,7 +355,7 @@ parse_error:
  * @src_fmt: the source format
  * @src_val: the source value
  * @dest_fmt: the destination format
- * @dest_val: a location to store the converted value
+ * @dest_val: (out): a location to store the converted value
  *
  * Convert @src_val in @src_fmt to the equivalent value in @dest_fmt. The result
  * will be put in @dest_val.
@@ -1670,10 +1670,10 @@ not_started:
  * gst_audio_ring_buffer_commit:
  * @buf: the #GstAudioRingBuffer to commit
  * @sample: the sample position of the data
- * @data: the data to commit
+ * @data: (array length=in_samples): the data to commit
  * @in_samples: the number of samples in the data to commit
  * @out_samples: the number of samples to write to the ringbuffer
- * @accum: accumulator for rate conversion.
+ * @accum: (inout): accumulator for rate conversion.
  *
  * Commit @in_samples samples pointed to by @data to the ringbuffer @buf.
  *
@@ -1723,9 +1723,9 @@ gst_audio_ring_buffer_commit (GstAudioRingBuffer * buf, guint64 * sample,
  * gst_audio_ring_buffer_read:
  * @buf: the #GstAudioRingBuffer to read from
  * @sample: the sample position of the data
- * @data: where the data should be read
+ * @data: (array length=len): where the data should be read
  * @len: the number of samples in data to read
- * @timestamp: where the timestamp is returned
+ * @timestamp: (out): where the timestamp is returned
  *
  * Read @len samples from the ringbuffer into the memory pointed
  * to by @data.
@@ -1826,7 +1826,7 @@ gst_audio_ring_buffer_read (GstAudioRingBuffer * buf, guint64 sample,
       /* Reorder from device order to GStreamer order */
       for (i = 0; i < sampleslen; i++) {
         for (j = 0; j < channels; j++) {
-          memcpy (data + reorder_map[j] * bps, ptr + j * bps, bps);
+          memcpy (data + i * bpf + reorder_map[j] * bps, ptr + j * bps, bps);
         }
         ptr += bpf;
       }
@@ -1843,7 +1843,7 @@ gst_audio_ring_buffer_read (GstAudioRingBuffer * buf, guint64 sample,
 
   if (buf->timestamps && timestamp) {
     *timestamp = buf->timestamps[readseg % segtotal];
-    GST_INFO_OBJECT (buf, "Retrieved timestamp %" GST_TIME_FORMAT
+    GST_DEBUG_OBJECT (buf, "Retrieved timestamp %" GST_TIME_FORMAT
         " @ %d", GST_TIME_ARGS (*timestamp), readseg % segtotal);
   }
 
@@ -1860,9 +1860,10 @@ not_started:
 /**
  * gst_audio_ring_buffer_prepare_read:
  * @buf: the #GstAudioRingBuffer to read from
- * @segment: the segment to read
- * @readptr: the pointer to the memory where samples can be read
- * @len: the number of bytes to read
+ * @segment: (out): the segment to read
+ * @readptr: (out) (array length=len):
+ *     the pointer to the memory where samples can be read
+ * @len: (out): the number of bytes to read
  *
  * Returns a pointer to memory where the data from segment @segment
  * can be found. This function is mostly used by subclasses.
@@ -2014,7 +2015,7 @@ position_less_channels (const GstAudioChannelPosition * pos, guint channels)
 /**
  * gst_audio_ring_buffer_set_channel_positions:
  * @buf: the #GstAudioRingBuffer
- * @position: the device channel positions
+ * @position: (array): the device channel positions
  *
  * Tell the ringbuffer about the device's channel positions. This must
  * be called in when the ringbuffer is acquired.
@@ -2083,7 +2084,7 @@ gst_audio_ring_buffer_set_timestamp (GstAudioRingBuffer * buf, gint readseg,
 {
   g_return_if_fail (GST_IS_AUDIO_RING_BUFFER (buf));
 
-  GST_INFO_OBJECT (buf, "Storing timestamp %" GST_TIME_FORMAT
+  GST_DEBUG_OBJECT (buf, "Storing timestamp %" GST_TIME_FORMAT
       " @ %d", GST_TIME_ARGS (timestamp), readseg);
 
   GST_OBJECT_LOCK (buf);
