@@ -19,21 +19,18 @@
 
 /**
  * SECTION:gstpbutilsdescriptions
+ * @title: Descriptions
  * @short_description: Provides human-readable descriptions for caps/codecs
  * and encoder, decoder, URI source and URI sink elements
  *
- * <refsect2>
- * <para>
  * The above functions provide human-readable strings for media formats
  * and decoder/demuxer/depayloader/encoder/muxer/payloader elements for use
  * in error dialogs or other messages shown to users.
- * </para>
- * <para>
+ *
  * gst_pb_utils_add_codec_description_to_tag_list() is a utility function
  * for demuxer and decoder elements to add audio/video codec tags from a
  * given (fixed) #GstCaps.
- * </para>
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -193,6 +190,7 @@ static const FormatInfo formats[] = {
   {"video/x-4xm", "4X Technologies Video", FLAG_VIDEO, ""},
   {"video/x-apple-video", "Apple video", FLAG_VIDEO, ""},
   {"video/x-aasc", "Autodesk Animator", FLAG_VIDEO, ""},
+  {"video/x-av1", "AV1", FLAG_VIDEO, ""},
   {"video/x-camtasia", "TechSmith Camtasia", FLAG_VIDEO, ""},
   {"video/x-cavs", "Chinese AVS (CAVS)", FLAG_VIDEO, ""},
   {"video/x-cdxa", "RIFF/CDXA (VCD)", AV_CONTAINER, ""},
@@ -286,6 +284,8 @@ static const FormatInfo formats[] = {
       ""},
   {"application/x-teletext", "Teletext", 0, ""},
   {"application/x-kate", "Kate", 0, ""},
+  {"closedcaption/x-cea-608", N_("CEA 608 Closed Caption"), FLAG_SUB, ""},
+  {"closedcaption/x-cea-708", N_("CEA 708 Closed Caption"), FLAG_SUB, ""},
   {"subtitle/x-kate", N_("Kate subtitle format"), FLAG_SUB, ""},
   {"application/x-subtitle-vtt", N_("WebVTT subtitle format"), FLAG_SUB, ""},
   {"subpicture/x-dvb", "DVB subtitles", FLAG_SUB, ""},
@@ -437,6 +437,8 @@ format_info_get_desc (const FormatInfo * info, const GstCaps * caps)
 
   g_assert (info != NULL);
 
+  gst_pb_utils_init_locale_text_domain ();
+
   if (info->desc != NULL)
     return g_strdup (_(info->desc));
 
@@ -822,11 +824,12 @@ format_info_get_desc (const FormatInfo * info, const GstCaps * caps)
     gint depth = 0;
     gboolean is_float;
     const gchar *str;
-    GstAudioFormat format;
+    GstAudioFormat format = GST_AUDIO_FORMAT_UNKNOWN;
     const GstAudioFormatInfo *finfo;
 
     str = gst_structure_get_string (s, "format");
-    format = gst_audio_format_from_string (str);
+    if (str)
+      format = gst_audio_format_from_string (str);
     if (format == GST_AUDIO_FORMAT_UNKNOWN)
       return g_strdup (_("Uncompressed audio"));
 
@@ -933,6 +936,8 @@ gst_pb_utils_get_source_description (const gchar * protocol)
 
   g_return_val_if_fail (protocol != NULL, NULL);
 
+  gst_pb_utils_init_locale_text_domain ();
+
   if (strcmp (protocol, "cdda") == 0)
     return g_strdup (_("Audio CD source"));
 
@@ -1022,6 +1027,8 @@ gst_pb_utils_get_decoder_description (const GstCaps * caps)
 
   g_return_val_if_fail (gst_caps_is_fixed (tmp), NULL);
 
+  gst_pb_utils_init_locale_text_domain ();
+
   /* special-case RTP caps */
   if (caps_are_rtp_caps (tmp, "video", &str)) {
     ret = g_strdup_printf (_("%s video RTP depayloader"), str);
@@ -1072,6 +1079,7 @@ gst_pb_utils_get_encoder_description (const GstCaps * caps)
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
   tmp = copy_and_clean_caps (caps);
   g_return_val_if_fail (gst_caps_is_fixed (tmp), NULL);
+  gst_pb_utils_init_locale_text_domain ();
 
   /* special-case RTP caps */
   if (caps_are_rtp_caps (tmp, "video", &str)) {
@@ -1120,6 +1128,8 @@ gst_pb_utils_get_element_description (const gchar * factory_name)
 
   g_return_val_if_fail (factory_name != NULL, NULL);
 
+  gst_pb_utils_init_locale_text_domain ();
+
   ret = g_strdup_printf (_("GStreamer element %s"), factory_name);
   if (ret && g_str_has_prefix (ret, factory_name))
     *ret = g_ascii_toupper (*ret);
