@@ -18,7 +18,9 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#include <unistd.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <gst/check/gstcheck.h>
 #include <gst/video/video-overlay-composition.h>
